#if !defined(_CCF_H)
#define _CCF_H

#ifndef _OS_DECTSTD_H
#include "os_dectstd.h"
#endif

#ifndef _OS_TIMER_H
#include "os_timer.h"
#endif

#ifndef _OS_MESSAGE_H
#include "os_message.h"
#endif

#ifndef _OS_QUEUE_H
#include "os_queue.h"
#endif

#ifndef _DSS1_L2_H
#include "dss1_l2.h"
#endif

#ifndef _FALC2256_H
/*#include "e1_falc.h"*/
#include "e1_falc2256.h"
#endif

#define MaxCell 16	/*16 cells*/ 
#define MaxMBC	64	/*16 cells * 4 bearers */ 
/*#define MaxADPCM	32*/	/*32 ADPCM channels */ 
#define MaxConnections	400	/*maximum one direction connections */ 
#define MceiOffset	0xa00

/********************************************/
enum Forward
{ 
 ColdStart,
 HotStart
}; 


/********************************************/
struct Uchannel
{
  struct DQE dqe;
  int ch;  /*for cell (0-3)*/
};
typedef struct Uchannel* pntrCh;

struct ADPCMchannel
{
  struct DQE dqe;
  int ch;  /*(0-31)*/
};
typedef struct ADPCMchannel* pntrAdpcmCh;


struct ConnectIdentifier
{
  struct DQE dqe;
  int id;  /*(0-0xff)*/
};
 typedef struct ConnectIdentifier* pntrId;

struct E1slot
{
 int PortSlot;
 pntrId pntr;
};


struct BearerInfo
{
  struct Cell* cell;
  int bearer_nb;
  pntrCh pntrUch;
  struct CFmidPmidStamp FmidPmidStamp;
};

/********************************************/
enum MBCstate
{
 MBCidle,	/* not used */
 MBCS0,		/* got a bearer setup request, waiting for confirmation from upper layer */
 MBCS1,		/* confirm sent, waiting for Bearer_established */
 MBCS2,		/* establish sent, waiting for cfm/rej */
 MBCW,
 MBCH,		/* handover first phase: ho_bearer setup */
 MBCH1,		/* handover special case: ho_bearer setup while main bearer released */
 MBCHT,		/* handover intermediate phase: timer for ho_bearer after it was established:*/
		/* PP decides it later -> ho may be still released by PP */
 MBCO,		/* handover second phase */
 MBCR		
};

/*********************************************************/
struct MBC
{
  struct MBC* pmbc;
  BYTE CreationCommand;

  int Uchannel_FP;

  WORD nb;
  struct BearerInfo main_bearer,ho_bearer;

  enum MBCstate state;
  DWORD Pmid;
  BOOL lc;
    /*WORD PCMchannelHo;*/
  BOOL RxCtPacket, TxCtPacket;
  struct Timer Tr;
  struct Timer BHT;

  struct DQ DeferredBearerInfo;

  pMessage TxFrame;
  BOOL FaxModem;
    /* this comes from Lc: */
  WORD FLEN;
  WORD RxByteCount;
  
  pMessage RxMess;
  BYTE* RxFrame;      
  
  WORD CreationCellNumber;   

  int mcei;
  int InSlot,InPort;
  int OutSlot,OutPort;
  pntrId pIdIn;
  int InSrc,InDst;
  pntrId pIdOut;
  int OutSrc,OutDst;

  pntrAdpcmCh pADPCM;

};

typedef struct MBC* pMBC;

/************ MAC Broadcast message control ************/
struct PageReqType
{
  BYTE Pid[2];
  int LifeTime;
};

/*********************************/
struct FifoPool
{
 int items;
 int size;
 struct PageReqType* data[6];
 int gp;
 int pp;
};

typedef struct FifoPool* pFPool;

/**************************************/
struct BMC
{
 struct DQ DeferredPageReqs;
 struct FifoPool PageReqUsed;
 struct FifoPool PageReqFree;
 struct PageReqType PageReq[6];
 int TailLifeTime;
 struct Timer T160;
 int MultiframeCounter;
};

/*------------------------------------------------------*/
enum CS
  {
   CellIdle,			/* waiting for cold or hot start */
   /*CellWaitEraseEnd,
   CellBootInProgress,
   CellWaitLoadResult,
   CellWaitStartInd,
   CellWaitDeactivation,
   CellWaitDummy,*/		/* cell fully loaded, we are waiting Dummy info */
   CellActive			/* cell fully loaded and functional */
  };

struct Cell
{
 enum CS cell_state;
 int DectCellNum;
/*
 WORD diagnostics;
*/

 struct MBC* mbc[12];

 /*statistics:*/
 int max_load,load;           
 DWORD load_array[13], load_tick;
 
 WORD line_nb; 
 BOOL release_mes;
};


/******************************************/ 
struct Lce
{
   enum
   {
    LinkReleased,
    LinkEstablished,
    EstablishPending,
    ReleasePending,
   /*additional class B link states*/
    LinkSuspended,
    SuspendPending,
    ResumePending
   }
    DatalinkState;

   struct DQ DeferredUpMessages;
};

struct LcManager
{
 int line_nb;
 struct Lce lce;
};
 
/*********************************/
enum PriLineType
{
 PriLineUndefined,
 PriLineDSS,
 PriLineCell,
 PriLineCluster0,
 PriLineCluster4, 
 PriLineFP0,
 PriLineFP4
};

struct PriLineTask
{           
 struct DQE dqe;
 
 int FreeBchannelCount;
 int nb;
 BOOL master;
 struct FALC* Falc;					/***Task* Falc;***/
 BOOL DL_isUser;
 struct DSS1Lapd* Lapd;				/***Task* Lapd;***/
 int* Lce;			/* ATTENTION!!! */	/***Task* Lce;***/	
 int PcmPort;			/* referenced to switch */
 BOOL isActive; 
 enum PriLineType type;
 BOOL TraceOn;
};


enum BriLineType
{
 BriLineUndefined,
 BriLineDSS,
 BriLineCell
};

struct BriLineTask
{           
 struct DQE dqe;
 int nb;
 
 int FreeBchannelCount;
 BOOL master;
 BOOL DL_isUser;
 struct DSS1Lapd* Lapd;				/***Task* Lapd;***/
 int* Lce;							/***Task* Lce;***/	
 int PcmPort;						/* referenced to switch */
 BOOL isActive; 
 enum BriLineType type;
 BOOL TraceOn;
};

/****************************************/
struct cellStore
{
 int size;
 int items;
 struct Cell* data[MaxCell];
};

struct mbcStore
{
 int size;
 int items;
 struct MBC* data[MaxMBC];
};

typedef struct cellStore* pcStore;
typedef struct mbcStore* pmStore;

/*******************************************/
struct LocalCluster
{
// union StandardCapabilities StdCaps;
// union StaticSystemInfo StatSysInfo;
 struct cellStore cells;	/***Store<Cell*> cells;***/
 struct mbcStore mbcs;		/***Store<MBC*> mbc;***/    
};

/*****************************************/

#endif
