#include <std.h>
#include <log.h>
#include <swi.h>
#include <hwi.h>
#include <string.h>
#include "os_mt.h"
/*#include "os_dectdefs.h"*/
#include "os_queue.h"
#include "os_timer.h"

struct TimerReel Timers1;
struct TimerReel Timers2;
struct TimerReel Timers3;
struct TimerReel TimersLLME;
/*struct TimerReel* pTimers;*/

extern enum StatusBreak Stop;
void StopMessage(enum StatusBreak st);

void initTimerReels()	/* incremental double-linked queue*/
{
 struct TimerReel* pTimers;

 pTimers=&Timers1;
 pTimers->entry=pTimers->exit=pTimers->tick_pointer=NULL;
 pTimers->items=0;

 pTimers=&Timers2;
 pTimers->entry=pTimers->exit=pTimers->tick_pointer=NULL;
 pTimers->items=0;

}

void TimerReelAudit(struct TimerReel* pTimers)
{
 if((pTimers->items==0)&&(pTimers->entry==NULL)&&
    (pTimers->exit==NULL))  return;
 if((pTimers->items!=0)&&(pTimers->entry!=NULL)&&
    (pTimers->exit!=NULL)) return;
 if((pTimers->items==1)&&(pTimers->entry==pTimers->exit)&&
 ((pTimers->tick_pointer==NULL)||
  (pTimers->tick_pointer==pTimers->entry)))
    return;
 StopMessage(TimerAudit);   /*Timer Reel audit*/
}

void TimerReelPut(ptimer t,struct TimerReel* pTimers)             
{                                   
 ptimer c;

 t->increment=t->period;

 if(pTimers->tick_pointer==NULL)	/* all expired or reel is empty*/
 {
  if(pTimers->exit==NULL)		/*empty reel*/
   pTimers->exit=t;
  else
   pTimers->entry->prev=t;      /*all expired, reel not empty*/

  t->next=pTimers->entry;
  pTimers->tick_pointer=pTimers->entry=t;
  t->prev=NULL;
  pTimers->items++;
  TimerReelAudit(pTimers);
  return;
 }

 else	/* timer_tick !=0: reel not empty, partially ticking*/
 {
  c=pTimers->tick_pointer;
  while(c->increment <= t->increment)
  {
   t->increment-=c->increment;
   c=c->prev;
   if(c==NULL) break;
  }

/* here we are: if c==null ->we are last
   else not last - we also could be at the (exit) of list*/

  if(c==NULL)
  {
   t->prev=NULL;
   t->next=pTimers->entry;
   pTimers->entry->prev=t;
   pTimers->entry=t;
  }

  else	/* we are not the last, (may be first)*/
  {
   c->increment-=t->increment;
   t->prev=c;
   t->next=c->next;
   if(c==pTimers->exit) pTimers->exit=t;  /*when first*/
   else
   {
    c->next->prev=t;
   }
   if(c==pTimers->tick_pointer) pTimers->tick_pointer=t;

   c->next=t;
  }
  pTimers->items++;
  TimerReelAudit(pTimers);
  return;

 } /* end else (not empty, ticking reel)*/
}

/*rewrite*/
void TimerReelTick()
{
 int i;
 struct TimerReel* pTimers;

 for(i=0;i<2;i++)    /*no TimersLLME !!!*/
 {
  switch(i)
  {
   case 0:
    pTimers=&Timers1;
   break;
   case 1:
    pTimers=&Timers2;
   break;
  }
  if(pTimers->tick_pointer==NULL)
  {
   TimerReelAudit(pTimers);
   continue;
/*   return;*/
  }
  if((--(pTimers->tick_pointer->increment))==0)	/* move tick_pointer*/
  {
   do
   {
    pTimers->tick_pointer=pTimers->tick_pointer->prev;
    if(pTimers->tick_pointer==NULL)
    {
     TimerReelAudit(pTimers);
     break;
/*     return;*/
    }
   }
   while(pTimers->tick_pointer->increment==0);

   TimerReelAudit(pTimers);
  }
 }/*loop for all TimerReels*/

}

ptimer TimerReelGetExpired(struct TimerReel* pTimers)
{
 ptimer result;

 HWI_disable();

 result=pTimers->exit;
 if(!(pTimers->items))
 {
  TimerReelAudit(pTimers);
  HWI_enable();
  return result;	/* NULL, empty reel*/
 }
 if(pTimers->exit->increment==0)
 {
  result->isRunning=FALSE;
  if(!(--(pTimers->items)))
  {
   pTimers->exit=pTimers->entry=NULL;
   TimerReelAudit(pTimers);

   HWI_enable();
   return result;
  }
  pTimers->exit=pTimers->exit->prev;
  pTimers->exit->next=NULL;
  TimerReelAudit(pTimers);
  HWI_enable();
  return result;
 }
 TimerReelAudit(pTimers);
 HWI_enable();
 return NULL;
}


/*rewrite*/
void TimerReelExtract(ptimer t,struct TimerReel* pTimers)
{
 ptimer c;
 if(pTimers->items==0)
 {
  TimerReelAudit(pTimers);
  return;
 }
 c=pTimers->entry;
 while(c!=t)
 {
  c= c->next;
  if(c==NULL)
  {
   TimerReelAudit(pTimers);
   return;	/* not found*/
  }
 }
 /* found*/

 if(t!=pTimers->exit)
  t->next->prev=t->prev;
 else
  pTimers->exit=t->prev;

 if(t!=pTimers->entry)
 {
  t->prev->next=t->next;
  if(t->increment>0)
   t->prev->increment+=t->increment;
 }
 else
  pTimers->entry=t->next;  /* t->prev=NULL earlier*/

 if(pTimers->tick_pointer==t)
  pTimers->tick_pointer=t->prev;
 pTimers->items--;

 TimerReelAudit(pTimers);

}

/************************************************
	Some timer functions:
*************************************************/
/*rewrite*/
void TimerStart(ptimer p,struct TimerReel* pTimers)
{
 if(p->isRunning)
        StopMessage(TimerAudit); /* Start of already running timer!*/

 TimerReelPut(p,pTimers);
 p->isRunning=TRUE;
}

/*rewrite*/
void TimerStop(ptimer p,struct TimerReel* pTimers)
{
 TimerReelExtract(p,pTimers);
 p->isRunning=FALSE;
 p->increment=0;
}

