#ifndef _FALC2256_H
#define _FALC2256_H

#ifndef _DECTSTD_H
#include "os_dectstd.h"
#endif

#ifndef _TIMER_H
#include "os_timer.h"
#endif

#ifndef _QUEUE_H
#include "os_queue.h"
#endif


#ifndef _MESSAGE_H
#include "os_message.h"
#endif


#include "os_mt.h"


#define TIME_SERVICE 20/*50mks(interrupt)*20=1ms*/
#define FALC_NUMBER 1
#define FALC_OFFSET0 0x400000
#define FALC_OFFSET1 0x408000


union FALC_FRS0
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL los:1;	/* loss of signal*/
  BOOL ais:1;	/* alarm indication signal*/
  BOOL lfa:1;	/* loss of frame alignment*/
  BOOL rra:1;	/* receive remote alarm*/
  BOOL res1:1;
  BOOL nmf:1;	/* no multiframe alignment found*/
  BOOL lmfa:1;	/* loss of multiframe alignment*/
  BOOL reserved:1;
 }s;
};

union FALC_FRS1
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL exzd:1;      /* excessive zeros detected*/
  BOOL ts16ra:1;	/* receive ts 16 remote alarm*/
  BOOL ts16los:1;	/* receive ts 16 loss of signal*/
  BOOL ts16ais:1;	/* receive ts 16 alarm indication*/
  BOOL ts16lfa:1;
  BOOL reserved2:1;
  BOOL xls:1;		/* tx line short*/
  BOOL xlo:1;		/* tx line open*/
 }s;
};


union FALC_ISR0
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL rme:1;	/* hdlc receive message end*/
  BOOL rfs:1;	/* hdlc receive frame start*/
  BOOL t8ms:1;	/* rx timeout 8 msec*/
  BOOL rmb:1;	/* rx multiframe begin*/
  BOOL casc:1;	/* rx cas info changed*/
  BOOL crc4:1;	/* rx crc4 error*/
  BOOL sa6sc:1;	/* rx sa6 status change*/
  BOOL rpf:1;	/* hdlc rx pool full*/
 }s;
};


union FALC_ISR1
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL llbsc:1;	/* line loop back status change*/
  BOOL rdo:1;	/* rx data overflow*/
  BOOL alls:1;	/* tx all sent*/
  BOOL xdu:1;	/* tx data underrun*/
  BOOL xmb:1;	/* transmit multiframe begin*/
  BOOL suex:1;	/* error threshold exceeded, only for SS7*/
  BOOL xlsc:1;	/* tx line status change*/
  BOOL xpr:1;	/* hdlc tx pool ready*/
 }s;
};

union FALC_ISR2
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL FAR:1;	/* frame alignment recovery*/
  BOOL lfa:1;	/* loss of frame alignment*/
  BOOL mfar:1;	/* multiframe alignment recovery*/
  BOOL t400ms:1;/* rx timeout*/
  BOOL ais:1;	/* alarm indication signal*/
  BOOL los:1;	/* loss of signal*/
  BOOL rar:1;	/* remote alarm recovery*/
  BOOL ra:1;	/* remote alarm*/
 }s;
};


union FALC_ISR3
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL es:1;	/* errored second*/
  BOOL sec:1;
  BOOL lmfa16:1;/* cas*/
  BOOL ais16:1;	/* cas*/
  BOOL ra16:1;	/* cas*/
  BOOL api:1;	/* aux pattern indication*/
  BOOL rsn:1;	/* receive slip negative*/
  BOOL rsp:1;	/* receive slip positive*/
 }s;
};

union FALC_ISR4
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL xsp:1;	/**/
  BOOL xsn:1;
  BOOL rme2:1;  /**/
  BOOL rfs2:1;	/**/
  BOOL rdo2:1;	/**/
  BOOL alls2:1;	/**/
  BOOL xdu2:1;	/**/
  BOOL rpf2:1;	/**/
 }s;
};


union FALC_ISR5
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL xpr2:1;	/**/
  BOOL xpr3:1;
  BOOL rme3:1;  /**/
  BOOL rfs3:1;	/**/
  BOOL rdo3:1;	/**/
  BOOL alls3:1;	/**/
  BOOL xdu3:1;	/**/
  BOOL rpf3:1;	/**/
 }s;
};

union FALC_IERR
{
 BYTE uc;
 struct
 {
  WORD unused:10;
  BOOL ifase:1; /* insert single FAS defect*/
  BOOL imfe:1;  /* insert single multiframe defect*/
  BOOL icrce:1;	/* insert single CRC defect*/
  BOOL icase:1;	/* insert single CAS defect*/
  BOOL ipe:1;	/* insert single PRBS defect*/
  BOOL ibv:1;	/* insert bipolar violation*/
 }s;
};

union FALC_FMR0
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  WORD xc:2;    /* transmit code: 11= HDB3*/
  WORD rc:2;    /* receive code: 11= HDB3*/
  BOOL extd:1;	/* extended HDB3 error detection = mode*/
  BOOL alm:1;	/* alarm mode, 0: AIS according to ETS 1: ITU 775 = mode*/
  BOOL frs:1;	/* force resync, 1-active, command*/
  BOOL sim:1;	/* alarm simulation, 0 -normal = command*/
 }s;
};


union FALC_FMR1
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL mfcs:1;	/* multiframe force resync*/
  BOOL afr:1;	/* auto force resync = mode*/
  BOOL ensa:1;	/* CAS related*/
  BOOL pmod:1;	/* PCM 30 mode=0*/
  BOOL xfs:1;	/* transmit framing select 1=CRC4*/
  BOOL ecm:1;	/* error counter mode 1: every second*/
  BOOL ssd0:1;	/* ssd1 in register SIC1!*/
  BOOL xais:1;	/* transmit AIS towards remote end = command*/
 }s;
};

union FALC_FMR2
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  WORD rfs:2; 	/* receive framing select*/
  BOOL rtm:1;	/* receive transparent mode*/
  BOOL dais:1;	/* disable AIS insertion into the system*/
  BOOL sais:1;	/* send AIS into the system*/
  BOOL plb:1;	/* payload loopback: 0- normal*/
  BOOL axra:1;	/* auto transmit remote alarm: 0=normal*/
  BOOL almf:1;	/* auto loss of multiframe: 0=normal*/
 }s;
};


union FALC_LIM0
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL xfb:1; 	/* transmit full bauded mode*/
  BOOL xdos:1;	/* transmit data out sense*/
  WORD reserved:2;	/* select clock output*/
  BOOL eqon:1;	/* receive equalizer ON*/
  BOOL rlm:1;	/* receive line monitoring,0-normal,1-monitoring*/
  BOOL ll:1;	/* local loop, 1=active*/
  BOOL mas:1;	/* master=1, slave=0*/
 }s;
};

union FALC_LCR1
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL eprm:1;	/* transmit data out sense*/
  BOOL xprbs:1;	/* transmit data out sense*/
  WORD ldc:2;	/* receive equalizer ON*/
  WORD lac:2;	/* enable loss of signal 0: clock free-running*/
  BOOL fllb:1;	/* local loop, 1=active*/
  BOOL llbp:1;	/* master=1, slave=0*/
 }s;
};

union FALC_SIC1
{                                                        
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL ssc1:1;
  BOOL ssd1:1;
  WORD rbs:2;	/* */
  BOOL ssc0:1;	/* */
  BOOL bim:1;   /*0- byte interleaved, 1- bit interleaved*/
  WORD xbs:2;	/* */
 }s;
};

union FALC_SIC2
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL ffs:1;		/**/
  BOOL ssf:1;		/**/
  BOOL srb:1;		/**/
  BOOL reserv1:1;	/**/
  WORD sics:3;		/**/
  BOOL reserv0:1;	/**/
 }s;
};

union FALC_CMR1
{
 BYTE uc;
 struct
 {
  WORD unused:10;
  WORD rs:2;	/* select rclk source*/
  BOOL dcs:1;	/* disable clock switching*/
  BOOL stf:1;	/* select tclk frequency*/
  BOOL dxja:1;	/* disable internal trnsm jitter attenuation*/
  BOOL dxss:1;	/* dco-x syncronization clock source*/
 }s;
};

union FALC_CMR2
{
 BYTE uc;
 struct
 {
  WORD unused:10;
  BOOL dcoxc:1;	/* dco-x center-frequency enable*/
  BOOL dcf:1;	/* dco-r center-frequency disable*/
  BOOL irsp:1;	/* internal rcv system frame sync pulse*/
  BOOL irsc:1;	/* internal rcv system clock*/
  BOOL ixsp:1;	/* internal trnsm system frame sync pulse*/
  BOOL ixsc:1;	/* internal trnsm system clock*/
 }s;
};

union FALC_GCR
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL vis:1;	/* masked int visible*/
  BOOL sci:1;	/* status change int*/
  BOOL ses:1;	/* select external second timer*/
  BOOL ecmc:1;	/* error counter mode COFA*/
  WORD reserv:3;
  BOOL pd:1;	/* power mode*/
 }s;
};

/* 1 in bit position of ECM deactivates
 the related second interrupt(ISR3.ES) */
union FALC_ECM
{
 BYTE uc;
 struct
 {
  WORD unused:8;
  BOOL lfa:1;
  BOOL fer:1;
  BOOL cer:1;
  BOOL ais:1;
  BOOL los:1;
  BOOL cve:1;
  BOOL slip:1;
  BOOL ebe:1;
 }s;
};

//version 2.1
union FALCchip
{
 struct
 {
  BYTE xfifo;
  BYTE xfifo1;
  BYTE cmdr;	/*2  hdlc*/
  BYTE mode;	/*3  hdlc*/
  BYTE rah1;	/*4  hdlc*/
  BYTE rah2;	/*5  hdlc*/
  BYTE ral1;	/*6  hdlc*/
  BYTE ral2;	/*7  hdlc*/
  BYTE ipc;		/*8  interrupt port configuration*/
  BYTE ccr1;	/*9  ph level config*/
  BYTE ccr2;	/*a  hdlc*/
  BYTE un0b;	/*0xb*/
  BYTE rtr[4];	/*c-f signalling slots*/
  BYTE ttr[4];	/*0x10-0x13 signalling slots*/

  BYTE imr0;	/*0x14*/
  BYTE imr1;	/*0x15*/
  BYTE imr2;	/*0x16*/
  BYTE imr3;	/*0x17*/
  BYTE imr4;	/*0x18*/
  BYTE imr5;	/*0x19*/

  BYTE un1a;	/*0x1a*/
  union FALC_IERR ierr;	/*0x1b*/

  union FALC_FMR0 fmr0;	/*0x1c*/
  union FALC_FMR1 fmr1;	/*0x1d*/
  union FALC_FMR2 fmr2;	/*0x1e*/
  BYTE loop;/*0x1f	loopback control, not used*/
  BYTE xsw;	/*0x20  transmit service word pulseframe*/
  BYTE xsp;	/*0x21  transmit spare bits: CAS enable/dis etc, some sync control over PH-level*/
  BYTE xc0;	/*0x22*/
  BYTE xc1;	/*0x23*/
  BYTE rc0;	/*0x24*/
  BYTE rc1;	/*0x25*/
  BYTE xpm0;	/*0x26 transmit pulse mode*/
  BYTE xpm1;	/*0x27*/
  BYTE xpm2;	/*0x28*/
  BYTE tswm;	/*0x29 not used*/
  BYTE un2a;	/*0x2a*/
  BYTE idle;	/*0x2b idle channel code register, not used*/
  BYTE xsa[5];	/*0x2c..0x30, not used*/
  BYTE fmr3;	/*0x31		not used */
  
  BYTE icb[4];	/*0x32..0x35 not used, idle channel register*/
  union FALC_LIM0 lim0;	/*0x36*/
  BYTE lim1;	/*0x37*/
  
  BYTE pcd;	/*0x38*/
  BYTE pcr;	/*0x39*/
  BYTE lim2;	/*0x3a not used*/
  union FALC_LCR1 lcr1;	/*0x3b*/
  BYTE lcr2;	/*0x3c*/
  BYTE lcr3;	/*0x3d*/
  
  union FALC_SIC1 sic1;	/*0x3e*/
  union FALC_SIC2 sic2;	/*0x3f*/
  BYTE sic3;	/*0x40*/

  BYTE un41[3];	/*0x41-0x43*/
  union FALC_CMR1 cmr1;	/*0x44*/
  union FALC_CMR2 cmr2;	/*0x45*/

  union FALC_GCR gcr;	/*0x46*/
  BYTE esm;	/*0x47*/

  BYTE cmr3;	 /*0x48-0x5f*/
  BYTE un49[23]; /*0x49-0x5f*/
  BYTE dec;	/*0x60	  to check!*/
  BYTE unused61[15];/*0x61-0x6f*/
  BYTE xs[16];  /*0x70..0x7f*/
  BYTE pc[5];   /*0x80..0x84*/
  BYTE gpc1;    /*0x85*/
  BYTE pc6;     /*0x86*/

  BYTE cmdr2;   /*0x87*/
  BYTE cmdr3;   /*0x88*/
  BYTE cmdr4;   /*0x89*/

  BYTE un8a;	/*0x8a*/
  
  BYTE ccr3;    /*0x8b*/
  BYTE ccr4;    /*0x8c*/
  BYTE ccr5;    /*0x8d*/

  BYTE mode2;   /*0x8e*/
  BYTE mode3;   /*0x8f*/
  
  BYTE un90[2]; /*0x90-0x91*/

  BYTE gcm[8];  /*0x92-0x99*/

  BYTE un9a[2]; /*0x9a-0x9b*/

  BYTE xfifo2;  /*0x9c*/
  BYTE xfifo21; /*0x9d*/
  BYTE xfifo3;  /*0x9e*/
  BYTE xfifo31; /*0x9f*/
  
  BYTE tseo; 	/*0xa0*/
  BYTE tsbs[3]; /*0xa1-0xa3*/
  BYTE tss[2];	/*0xa4-0xa5*/
  BYTE una6[2]; /*0xa6-0xa7*/
  BYTE tpc0;	/*0xa8*/

  BYTE glc1;	/*0xaf*/

 }w;
 struct
 {
  BYTE rfifo;
  BYTE rfifo1;
  BYTE unused2;	/*2*/
  BYTE mode;	/*3*/
  BYTE rah1;	/*4*/
  BYTE rah2;	/*5*/
  BYTE ral1;	/*6*/
  BYTE ral2;	/*7*/
  BYTE ipc;		/*8*/
  BYTE ccr1;	/*9*/
  BYTE ccr2;	/*a*/
  
  BYTE pre;		/*b unused*/
  BYTE rtr[4];	/*c-f*/
  BYTE ttr[4];	/*0x10-0x13*/
  BYTE imr0;	/*0x14*/
  BYTE imr1;	/*0x15*/
  BYTE imr2;	/*0x16*/
  BYTE imr3;	/*0x17*/
  BYTE imr4;	/*0x18*/
  BYTE imr5;	/*0x19*/

  BYTE un1a;	/*0x1a*/
  union FALC_IERR ierr;	/*0x1b*/

  union FALC_FMR0 fmr0;	/*0x1c*/
  union FALC_FMR1 fmr1;	/*0x1d*/
  union FALC_FMR2 fmr2;	/*0x1e*/
  BYTE loop;/*0x1f	loopback control, not used*/
  BYTE xsw;	/*0x20  transmit service word pulseframe*/
  BYTE xsp;	/*0x21  transmit spare bits: CAS enable/dis etc, some sync control over PH-level*/
  BYTE xc0;	/*0x22*/
  BYTE xc1;	/*0x23*/
  BYTE rc0;	/*0x24*/
  BYTE rc1;	/*0x25*/
  BYTE xpm0;	/*0x26 transmit pulse mode*/
  BYTE xpm1;	/*0x27*/
  BYTE xpm2;	/*0x28*/
  BYTE tswm;	/*0x29 not used*/
  BYTE un2a;	/*0x2a*/
  BYTE idle;	/*0x2b idle channel code register, not used*/
  BYTE xsa[5];	/*0x2c..0x30, not used*/
  BYTE fmr3;	/*0x31		not used */
  
  BYTE icb[4];	/*0x32..0x35 not used, idle channel register*/
  union FALC_LIM0 lim0;	/*0x36*/
  BYTE lim1;	/*0x37*/
  
  BYTE pcd;	/*0x38*/
  BYTE pcr;	/*0x39*/
  BYTE lim2;	/*0x3a not used*/
  union FALC_LCR1 lcr1;	/*0x3b*/
  BYTE lcr2;	/*0x3c*/
  BYTE lcr3;	/*0x3d*/
  
  union FALC_SIC1 sic1;	/*0x3e*/
  union FALC_SIC2 sic2;	/*0x3f*/
  BYTE sic3;	/*0x40*/

  BYTE un41[3];	/*0x41-0x43*/
  union FALC_CMR1 cmr1;	/*0x44*/
  union FALC_CMR2 cmr2;	/*0x45*/

  union FALC_GCR gcr;	/*0x46*/
  BYTE esm;	/*0x47*/
  
  BYTE un48;	/*0x48*/
  BYTE rbd;		/*0x49*/
  BYTE vstr;	/*0x4a*/
  BYTE res;		/*0x4b receive equalizer status*/
  union FALC_FRS0 frs0;	/*0x4c*/
  union FALC_FRS1 frs1;	/*0x4d*/
  BYTE rsw;     /*0x4e receive service word pulseframe, not used*/

  BYTE rsp;     /*0x4f receive spare bits, not used*/
  BYTE fec[2];	/*0x50*/
  BYTE cvc[2];  /*0x52= violations of hdb3 code counter*/
  BYTE cec1[2]; /*0x54 = CRC error counter*/
  BYTE ebc[2];  /*0x56 = E -bit error counter*/
  BYTE cec2[2]; /*0x58 = SA6 bit error counter*/
  BYTE cec3[2]; /*0x5a = SA6 bit error counter*/
  BYTE rsa[6];	/*0x5c..61*/
  BYTE rsp1;	/*0x62*/
  BYTE rsp2;	/*0x63*/

  BYTE sis;		/*0x64  hdlc*/
  BYTE rsis;	/*0x65  hdlc*/
  BYTE rbc[2];	/*0x66  hdlc*/
  union FALC_ISR0 isr0;	/*68*/
  union FALC_ISR1 isr1;	/*69*/
  union FALC_ISR2 isr2;	/*6a*/
  union FALC_ISR3 isr3;	/*6b*/
  union FALC_ISR4 isr4;	/*6c*/
  union FALC_ISR5 isr5;	/*6d*/

  BYTE gis;		/*6e*/
  BYTE un6f;	/*6f*/
  BYTE rs[16];	/*70..7f*/

  BYTE pc[5];   /*0x80..0x84*/
  BYTE gpc1;    /*0x85*/
  BYTE pc6;     /*0x86*/

  BYTE un87[3];	/*87..89*/
  BYTE un8a;	/*0x8a*/
  
  BYTE ccr3;    /*0x8b*/
  BYTE ccr4;    /*0x8c*/
  BYTE ccr5;    /*0x8d*/

  BYTE mode2;   /*0x8e*/
  BYTE mode3;   /*0x8f*/
  
  BYTE rbc2;	/*0x90*/
  BYTE rbc3;	/*0x91*/

  BYTE gcm[8];  /*0x92-0x99*/

  BYTE sis3; 	/*0x9a  hdlc*/
  BYTE rsis3;	/*0x9b  hdlc*/
  
  BYTE rfifo2;  /*0x9c*/
  BYTE rfifo21; /*0x9d*/
  BYTE rfifo3;  /*0x9e*/
  BYTE rfifo31; /*0x9f*/
  
  BYTE una0[9];	/*0xa0-0xa8*/
  BYTE sis2;	/*0xa9*/
  BYTE rsis2;	/*0xaa*/
  BYTE mfpi;	/*0xab*/

  //BYTE unac[54];	/*0xac-0xeb*/
  BYTE unac[64];	/*0xac-0xeb*/
  BYTE wid; 	/*0xec*/
 }r;
};

enum FalcState
{
 FalcIdle,
 FalcDelay,
 FalcSync,
 FalcDesync,
 FalcDesyncAlarm,	/* first 0.5 secs*/
 NotInit,
 PreInit
};

/*
enum  f_id
{
   LOS,
   LFA,
   LMFA,
   RA,
   line_ok
};
*/

enum f_id
{
   XS,
   LOS,
   LFA,
   AIS,
   AUXP,
   LMFA,
   RA,
   line_ok
};


enum FalcVersion
{
 FALC54_1V2=0,
 FALC54_1V3=1,
 FALC54_1V4=2,
 FALC55_1V1=0x10,
 FALC55_1V3=0x13
};

struct FALC
{
  BOOL MphConfigure;
  enum f_id fault_id;
  struct Timer tt;
  struct Timer T101;	/* 0.5 secs when Lower layer failed*/
  ptimer ptt;
  ptimer pT101;
  struct Timer T_INIT;	/* 0.1 s*/
  ptimer pT_INIT;

  enum FalcState state;
  BOOL TxReady;
  struct DQ DeferredDataReqs;
  union FALCchip* chip;

  pMessage TxMessage;
  int TxLenght;
  int TxIndex;

  BYTE RxMessage[280];
  int RxLength;
  BOOL crc4;
  BOOL long_haul;

  enum
  {
   sync_slave,
   sync_master,
   sync_smart
  }sync_mode;

 int cntr_crc4;
 int cntr_ebit;
 int hdlc_ch;
 int sa7;

 enum FalcVersion version;
};

extern struct FALC falc[FALC_NUMBER];

void Init_Falc(pMessage m);
BOOL L1_ok();
void Falc_DnProcess(pFlag m);
void Falc_UpProcess(pMessage m);
void TimeOutFalc(ptimer t);
void Transmit();
void WaitRsis();


#endif

